//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: AntSimulatorTab.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Threading;

namespace AntGP
{
	/// <summary>
	/// This control allows the user to view the ant program executing.
    /// The user can also make a custom map by placing items on a SimluatorControl.
	/// </summary>
	public class AntSimulatorTab : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.Button ClearMapButton;
		private System.Windows.Forms.ComboBox CannedMapComboBox;
		private System.Windows.Forms.Label NumberOfTurnsLabel;
		private System.Windows.Forms.Label FitnessLabel;
		private System.Windows.Forms.Label ProgramSizeLabel;
		private System.Windows.Forms.Label FoodNotCollectedLabel;
		private System.Windows.Forms.Label FoodNotLocatedLabel;
		private System.Windows.Forms.Label AntSpeedLabel;
		private System.Windows.Forms.TrackBar AntSpeedSlider;
		private System.Windows.Forms.Label PheromoneDissipationSpeedLabel;
		private System.Windows.Forms.TrackBar PheromoneDissipationSpeedSlider;
		private System.Windows.Forms.TrackBar NumberOfAntsSlider;
		private System.Windows.Forms.Label NumberOfAntsLabel;
		private System.Windows.Forms.Label LoadMapLabel;
		private System.Windows.Forms.Label PheromoneStrengthLabel;
		private System.Windows.Forms.TrackBar PheromoneStrengthSlider;
		private System.Windows.Forms.Button AddNestButton;
		private System.Windows.Forms.Button AddWaterButton;
		private System.Windows.Forms.Button AddObstacleButton;
		private System.Windows.Forms.Button AddFood1Button;
		private System.Windows.Forms.Button AddFood2Button;
		private System.Windows.Forms.Button AddFood3Button;
		private System.Windows.Forms.Button AddFood4Button;
		private System.Windows.Forms.Button AddFood5Button;
		private System.Windows.Forms.Button AddFood6Button;
		private System.Windows.Forms.Button AddFood7Button;
		private System.Windows.Forms.Button AddFood8Button;
		private System.Windows.Forms.Button AddFood9Button;
		private System.Windows.Forms.Label AddItemToMapLabel;
		private System.Windows.Forms.Label LoadSolutionLabel;
		private System.Windows.Forms.ComboBox LoadSolutionCombo;
		private System.Windows.Forms.Button ClearSquareButton;
		private System.Windows.Forms.Button TestSolutionButton;
        private System.Windows.Forms.Label WaitForHelpPercentageLabel;
        private System.Windows.Forms.TrackBar WaitForHelpPercentageSlider;
        private System.Windows.Forms.Button selectedMapItemButton = null;
        private System.Windows.Forms.Button AddTreeButton;
        private System.Windows.Forms.Label WaterRemainingLabel;
        private AntGP.SimulatorControl SimulatorArea;

		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

        /// <summary>
        /// Used to fetch the map icons from the AntGP.DLL.
        /// </summary>
		private Icons icons = new Icons();
		
        /// <summary>
        /// The program to run in the simulator.
        /// </summary>
        private Program program;

        /// <summary>
        /// The map to use for this simluaton.
        /// </summary>
		private Map map = new Map();
        
        /// <summary>
        /// Genetic Program Options from the Program Tab.
        /// These are only used to get the parameters for the fitness formula.
        /// </summary>
        private GeneticProgramOptions options;

        /// <summary>
        /// Default constructor.
        /// </summary>
		public AntSimulatorTab()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			AddIconsToMapItemButtons();

			DrawSelectedMapItem();

            CannedMapComboBox_SelectedIndexChanged(null,null);
            LoadSolutionCombo_SelectedIndexChanged(null, null);
		}
        
        /// <summary>
        /// This function puts the icons for each map item on the button that is used to place the item.
        /// </summary>
		private void AddIconsToMapItemButtons()
		{
			if ( !this.DesignMode ) // Do this so that the bitmaps don't end up in the .resx file.
			{
				this.ClearSquareButton.BackgroundImage = icons.GetIcon( MapItem.Item.Nothing, ClearSquareButton.Size );
                this.AddTreeButton.BackgroundImage = icons.GetIcon( MapItem.Item.Tree, AddTreeButton.Size );
				this.AddNestButton.BackgroundImage = icons.GetIcon( MapItem.Item.Nest, AddNestButton.Size );
				this.AddWaterButton.BackgroundImage = icons.GetIcon( MapItem.Item.Water, AddWaterButton.Size );
				this.AddObstacleButton.BackgroundImage = icons.GetIcon( MapItem.Item.Obstacle, AddObstacleButton.Size );
				this.AddFood1Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food1, AddFood1Button.Size );
				this.AddFood2Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food2, AddFood2Button.Size );
				this.AddFood3Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food3, AddFood3Button.Size );
				this.AddFood4Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food4, AddFood4Button.Size );
				this.AddFood5Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food5, AddFood5Button.Size );
				this.AddFood6Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food6, AddFood6Button.Size );
				this.AddFood7Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food7, AddFood7Button.Size );
				this.AddFood8Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food8, AddFood8Button.Size );
				this.AddFood9Button.BackgroundImage = icons.GetIcon( MapItem.Item.Food9, AddFood9Button.Size );
			}
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.ClearMapButton = new System.Windows.Forms.Button();
            this.LoadMapLabel = new System.Windows.Forms.Label();
            this.CannedMapComboBox = new System.Windows.Forms.ComboBox();
            this.NumberOfTurnsLabel = new System.Windows.Forms.Label();
            this.FitnessLabel = new System.Windows.Forms.Label();
            this.ProgramSizeLabel = new System.Windows.Forms.Label();
            this.FoodNotCollectedLabel = new System.Windows.Forms.Label();
            this.FoodNotLocatedLabel = new System.Windows.Forms.Label();
            this.AntSpeedLabel = new System.Windows.Forms.Label();
            this.AntSpeedSlider = new System.Windows.Forms.TrackBar();
            this.PheromoneDissipationSpeedLabel = new System.Windows.Forms.Label();
            this.PheromoneDissipationSpeedSlider = new System.Windows.Forms.TrackBar();
            this.NumberOfAntsSlider = new System.Windows.Forms.TrackBar();
            this.NumberOfAntsLabel = new System.Windows.Forms.Label();
            this.PheromoneStrengthLabel = new System.Windows.Forms.Label();
            this.PheromoneStrengthSlider = new System.Windows.Forms.TrackBar();
            this.AddNestButton = new System.Windows.Forms.Button();
            this.AddWaterButton = new System.Windows.Forms.Button();
            this.AddObstacleButton = new System.Windows.Forms.Button();
            this.AddFood1Button = new System.Windows.Forms.Button();
            this.AddFood2Button = new System.Windows.Forms.Button();
            this.AddFood3Button = new System.Windows.Forms.Button();
            this.AddFood4Button = new System.Windows.Forms.Button();
            this.AddFood5Button = new System.Windows.Forms.Button();
            this.AddFood6Button = new System.Windows.Forms.Button();
            this.AddFood7Button = new System.Windows.Forms.Button();
            this.AddFood8Button = new System.Windows.Forms.Button();
            this.AddFood9Button = new System.Windows.Forms.Button();
            this.AddItemToMapLabel = new System.Windows.Forms.Label();
            this.LoadSolutionLabel = new System.Windows.Forms.Label();
            this.LoadSolutionCombo = new System.Windows.Forms.ComboBox();
            this.WaitForHelpPercentageLabel = new System.Windows.Forms.Label();
            this.WaitForHelpPercentageSlider = new System.Windows.Forms.TrackBar();
            this.TestSolutionButton = new System.Windows.Forms.Button();
            this.ClearSquareButton = new System.Windows.Forms.Button();
            this.AddTreeButton = new System.Windows.Forms.Button();
            this.WaterRemainingLabel = new System.Windows.Forms.Label();
            this.SimulatorArea = new AntGP.SimulatorControl();
            ((System.ComponentModel.ISupportInitialize)(this.AntSpeedSlider)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.PheromoneDissipationSpeedSlider)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.NumberOfAntsSlider)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.PheromoneStrengthSlider)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.WaitForHelpPercentageSlider)).BeginInit();
            this.SuspendLayout();
            // 
            // ClearMapButton
            // 
            this.ClearMapButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.ClearMapButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.ClearMapButton.Location = new System.Drawing.Point(616, 416);
            this.ClearMapButton.Name = "ClearMapButton";
            this.ClearMapButton.Size = new System.Drawing.Size(152, 23);
            this.ClearMapButton.TabIndex = 7;
            this.ClearMapButton.Text = "Clear Map";
            this.ClearMapButton.UseVisualStyleBackColor = false;
            this.ClearMapButton.Click += new System.EventHandler(this.ClearMapButton_Click);
            // 
            // LoadMapLabel
            // 
            this.LoadMapLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.LoadMapLabel.Location = new System.Drawing.Point(616, 368);
            this.LoadMapLabel.Name = "LoadMapLabel";
            this.LoadMapLabel.Size = new System.Drawing.Size(176, 16);
            this.LoadMapLabel.TabIndex = 29;
            this.LoadMapLabel.Text = "Load Map";
            // 
            // CannedMapComboBox
            // 
            this.CannedMapComboBox.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.CannedMapComboBox.Font = new System.Drawing.Font("Verdana", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.CannedMapComboBox.ItemHeight = 13;
            this.CannedMapComboBox.Items.AddRange(new object[] {
            "Use Same Map from Tab1",
            "Three Food(1) Piles",
            "Six Weight Food(1,3,6) Piles",
            "Seven Food(6) Pieces",
            "Ten Food(9) Pieces",
            "One Food(1) Pile With River",
            "Three Food(1) Piles River Tree",
            "Six Food(5) Pieces Wall/River"});
            this.CannedMapComboBox.Location = new System.Drawing.Point(616, 392);
            this.CannedMapComboBox.Name = "CannedMapComboBox";
            this.CannedMapComboBox.Size = new System.Drawing.Size(176, 21);
            this.CannedMapComboBox.TabIndex = 6;
            this.CannedMapComboBox.Text = "Three Food(1) Piles";
            this.CannedMapComboBox.SelectedIndexChanged += new System.EventHandler(this.CannedMapComboBox_SelectedIndexChanged);
            // 
            // NumberOfTurnsLabel
            // 
            this.NumberOfTurnsLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.NumberOfTurnsLabel.Location = new System.Drawing.Point(616, 264);
            this.NumberOfTurnsLabel.Name = "NumberOfTurnsLabel";
            this.NumberOfTurnsLabel.Size = new System.Drawing.Size(176, 16);
            this.NumberOfTurnsLabel.TabIndex = 0;
            this.NumberOfTurnsLabel.Text = "Number of Turns:";
            // 
            // FitnessLabel
            // 
            this.FitnessLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.FitnessLabel.Location = new System.Drawing.Point(616, 344);
            this.FitnessLabel.Name = "FitnessLabel";
            this.FitnessLabel.Size = new System.Drawing.Size(176, 16);
            this.FitnessLabel.TabIndex = 0;
            this.FitnessLabel.Text = "Fitness:";
            // 
            // ProgramSizeLabel
            // 
            this.ProgramSizeLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.ProgramSizeLabel.Location = new System.Drawing.Point(616, 328);
            this.ProgramSizeLabel.Name = "ProgramSizeLabel";
            this.ProgramSizeLabel.Size = new System.Drawing.Size(176, 16);
            this.ProgramSizeLabel.TabIndex = 0;
            this.ProgramSizeLabel.Text = "Program Size:";
            // 
            // FoodNotCollectedLabel
            // 
            this.FoodNotCollectedLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.FoodNotCollectedLabel.Location = new System.Drawing.Point(616, 280);
            this.FoodNotCollectedLabel.Name = "FoodNotCollectedLabel";
            this.FoodNotCollectedLabel.Size = new System.Drawing.Size(176, 16);
            this.FoodNotCollectedLabel.TabIndex = 0;
            this.FoodNotCollectedLabel.Text = "Food Not Collected:";
            // 
            // FoodNotLocatedLabel
            // 
            this.FoodNotLocatedLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.FoodNotLocatedLabel.Location = new System.Drawing.Point(616, 296);
            this.FoodNotLocatedLabel.Name = "FoodNotLocatedLabel";
            this.FoodNotLocatedLabel.Size = new System.Drawing.Size(176, 16);
            this.FoodNotLocatedLabel.TabIndex = 0;
            this.FoodNotLocatedLabel.Text = "Food Not Located:";
            // 
            // AntSpeedLabel
            // 
            this.AntSpeedLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AntSpeedLabel.Location = new System.Drawing.Point(616, 56);
            this.AntSpeedLabel.Name = "AntSpeedLabel";
            this.AntSpeedLabel.Size = new System.Drawing.Size(176, 16);
            this.AntSpeedLabel.TabIndex = 0;
            this.AntSpeedLabel.Text = "Ant Movement Speed";
            this.AntSpeedLabel.TextAlign = System.Drawing.ContentAlignment.TopCenter;
            // 
            // AntSpeedSlider
            // 
            this.AntSpeedSlider.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AntSpeedSlider.Location = new System.Drawing.Point(616, 72);
            this.AntSpeedSlider.Name = "AntSpeedSlider";
            this.AntSpeedSlider.Size = new System.Drawing.Size(176, 45);
            this.AntSpeedSlider.TabIndex = 2;
            this.AntSpeedSlider.Value = 10;
            // 
            // PheromoneDissipationSpeedLabel
            // 
            this.PheromoneDissipationSpeedLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.PheromoneDissipationSpeedLabel.Location = new System.Drawing.Point(616, 152);
            this.PheromoneDissipationSpeedLabel.Name = "PheromoneDissipationSpeedLabel";
            this.PheromoneDissipationSpeedLabel.Size = new System.Drawing.Size(176, 16);
            this.PheromoneDissipationSpeedLabel.TabIndex = 0;
            this.PheromoneDissipationSpeedLabel.Text = "Pheromone Spread Speed";
            this.PheromoneDissipationSpeedLabel.TextAlign = System.Drawing.ContentAlignment.TopCenter;
            // 
            // PheromoneDissipationSpeedSlider
            // 
            this.PheromoneDissipationSpeedSlider.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.PheromoneDissipationSpeedSlider.Location = new System.Drawing.Point(616, 168);
            this.PheromoneDissipationSpeedSlider.Maximum = 100;
            this.PheromoneDissipationSpeedSlider.Name = "PheromoneDissipationSpeedSlider";
            this.PheromoneDissipationSpeedSlider.Size = new System.Drawing.Size(176, 45);
            this.PheromoneDissipationSpeedSlider.TabIndex = 4;
            this.PheromoneDissipationSpeedSlider.TickFrequency = 10;
            this.PheromoneDissipationSpeedSlider.Value = 40;
            // 
            // NumberOfAntsSlider
            // 
            this.NumberOfAntsSlider.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.NumberOfAntsSlider.Location = new System.Drawing.Point(616, 24);
            this.NumberOfAntsSlider.Maximum = 100;
            this.NumberOfAntsSlider.Minimum = 1;
            this.NumberOfAntsSlider.Name = "NumberOfAntsSlider";
            this.NumberOfAntsSlider.Size = new System.Drawing.Size(176, 45);
            this.NumberOfAntsSlider.TabIndex = 1;
            this.NumberOfAntsSlider.TickFrequency = 10;
            this.NumberOfAntsSlider.Value = 50;
            // 
            // NumberOfAntsLabel
            // 
            this.NumberOfAntsLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.NumberOfAntsLabel.Location = new System.Drawing.Point(616, 8);
            this.NumberOfAntsLabel.Name = "NumberOfAntsLabel";
            this.NumberOfAntsLabel.Size = new System.Drawing.Size(176, 16);
            this.NumberOfAntsLabel.TabIndex = 0;
            this.NumberOfAntsLabel.Text = "Number Of Ants";
            this.NumberOfAntsLabel.TextAlign = System.Drawing.ContentAlignment.TopCenter;
            // 
            // PheromoneStrengthLabel
            // 
            this.PheromoneStrengthLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.PheromoneStrengthLabel.Location = new System.Drawing.Point(616, 104);
            this.PheromoneStrengthLabel.Name = "PheromoneStrengthLabel";
            this.PheromoneStrengthLabel.Size = new System.Drawing.Size(176, 16);
            this.PheromoneStrengthLabel.TabIndex = 0;
            this.PheromoneStrengthLabel.Text = "Pheromone Strength";
            this.PheromoneStrengthLabel.TextAlign = System.Drawing.ContentAlignment.TopCenter;
            // 
            // PheromoneStrengthSlider
            // 
            this.PheromoneStrengthSlider.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.PheromoneStrengthSlider.Location = new System.Drawing.Point(616, 120);
            this.PheromoneStrengthSlider.Maximum = 1000;
            this.PheromoneStrengthSlider.Name = "PheromoneStrengthSlider";
            this.PheromoneStrengthSlider.Size = new System.Drawing.Size(176, 45);
            this.PheromoneStrengthSlider.TabIndex = 3;
            this.PheromoneStrengthSlider.TickFrequency = 100;
            this.PheromoneStrengthSlider.Value = 300;
            // 
            // AddNestButton
            // 
            this.AddNestButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddNestButton.Location = new System.Drawing.Point(664, 468);
            this.AddNestButton.Name = "AddNestButton";
            this.AddNestButton.Size = new System.Drawing.Size(24, 24);
            this.AddNestButton.TabIndex = 10;
            this.AddNestButton.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddWaterButton
            // 
            this.AddWaterButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddWaterButton.Location = new System.Drawing.Point(692, 468);
            this.AddWaterButton.Name = "AddWaterButton";
            this.AddWaterButton.Size = new System.Drawing.Size(24, 24);
            this.AddWaterButton.TabIndex = 11;
            this.AddWaterButton.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddObstacleButton
            // 
            this.AddObstacleButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddObstacleButton.Location = new System.Drawing.Point(720, 468);
            this.AddObstacleButton.Name = "AddObstacleButton";
            this.AddObstacleButton.Size = new System.Drawing.Size(24, 24);
            this.AddObstacleButton.TabIndex = 12;
            this.AddObstacleButton.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood1Button
            // 
            this.AddFood1Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood1Button.Location = new System.Drawing.Point(748, 468);
            this.AddFood1Button.Name = "AddFood1Button";
            this.AddFood1Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood1Button.TabIndex = 13;
            this.AddFood1Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood2Button
            // 
            this.AddFood2Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood2Button.Location = new System.Drawing.Point(776, 468);
            this.AddFood2Button.Name = "AddFood2Button";
            this.AddFood2Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood2Button.TabIndex = 14;
            this.AddFood2Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood3Button
            // 
            this.AddFood3Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood3Button.Location = new System.Drawing.Point(608, 496);
            this.AddFood3Button.Name = "AddFood3Button";
            this.AddFood3Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood3Button.TabIndex = 15;
            this.AddFood3Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood4Button
            // 
            this.AddFood4Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood4Button.Location = new System.Drawing.Point(636, 496);
            this.AddFood4Button.Name = "AddFood4Button";
            this.AddFood4Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood4Button.TabIndex = 16;
            this.AddFood4Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood5Button
            // 
            this.AddFood5Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood5Button.Location = new System.Drawing.Point(664, 496);
            this.AddFood5Button.Name = "AddFood5Button";
            this.AddFood5Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood5Button.TabIndex = 17;
            this.AddFood5Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood6Button
            // 
            this.AddFood6Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood6Button.Location = new System.Drawing.Point(692, 496);
            this.AddFood6Button.Name = "AddFood6Button";
            this.AddFood6Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood6Button.TabIndex = 18;
            this.AddFood6Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood7Button
            // 
            this.AddFood7Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood7Button.Location = new System.Drawing.Point(720, 496);
            this.AddFood7Button.Name = "AddFood7Button";
            this.AddFood7Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood7Button.TabIndex = 19;
            this.AddFood7Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood8Button
            // 
            this.AddFood8Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood8Button.Location = new System.Drawing.Point(748, 496);
            this.AddFood8Button.Name = "AddFood8Button";
            this.AddFood8Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood8Button.TabIndex = 20;
            this.AddFood8Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddFood9Button
            // 
            this.AddFood9Button.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddFood9Button.Location = new System.Drawing.Point(776, 496);
            this.AddFood9Button.Name = "AddFood9Button";
            this.AddFood9Button.Size = new System.Drawing.Size(24, 24);
            this.AddFood9Button.TabIndex = 21;
            this.AddFood9Button.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddItemToMapLabel
            // 
            this.AddItemToMapLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddItemToMapLabel.Location = new System.Drawing.Point(612, 448);
            this.AddItemToMapLabel.Name = "AddItemToMapLabel";
            this.AddItemToMapLabel.Size = new System.Drawing.Size(144, 16);
            this.AddItemToMapLabel.TabIndex = 38;
            this.AddItemToMapLabel.Text = "Add Items to Map:";
            // 
            // LoadSolutionLabel
            // 
            this.LoadSolutionLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.LoadSolutionLabel.Location = new System.Drawing.Point(616, 532);
            this.LoadSolutionLabel.Name = "LoadSolutionLabel";
            this.LoadSolutionLabel.Size = new System.Drawing.Size(168, 16);
            this.LoadSolutionLabel.TabIndex = 39;
            this.LoadSolutionLabel.Text = "Load Solution";
            // 
            // LoadSolutionCombo
            // 
            this.LoadSolutionCombo.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.LoadSolutionCombo.Font = new System.Drawing.Font("Verdana", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.LoadSolutionCombo.Items.AddRange(new object[] {
            "Use Last Evolved Solution",
            "Use Program From Tab 2",
            "Simple Light Grouped Food Solution",
            "Simple Heavy Food Solution",
            "River Solution w/ Dead Ants",
            "River Solution w/ Trees",
            "Heavy Food with River Solution"});
            this.LoadSolutionCombo.Location = new System.Drawing.Point(616, 548);
            this.LoadSolutionCombo.Name = "LoadSolutionCombo";
            this.LoadSolutionCombo.Size = new System.Drawing.Size(176, 21);
            this.LoadSolutionCombo.TabIndex = 22;
            this.LoadSolutionCombo.Text = "Simple Light Grouped Food Solution";
            this.LoadSolutionCombo.SelectedIndexChanged += new System.EventHandler(this.LoadSolutionCombo_SelectedIndexChanged);
            // 
            // WaitForHelpPercentageLabel
            // 
            this.WaitForHelpPercentageLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.WaitForHelpPercentageLabel.Location = new System.Drawing.Point(616, 200);
            this.WaitForHelpPercentageLabel.Name = "WaitForHelpPercentageLabel";
            this.WaitForHelpPercentageLabel.Size = new System.Drawing.Size(176, 16);
            this.WaitForHelpPercentageLabel.TabIndex = 41;
            this.WaitForHelpPercentageLabel.Text = "Wait for help Percentage";
            this.WaitForHelpPercentageLabel.TextAlign = System.Drawing.ContentAlignment.TopCenter;
            // 
            // WaitForHelpPercentageSlider
            // 
            this.WaitForHelpPercentageSlider.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.WaitForHelpPercentageSlider.Location = new System.Drawing.Point(616, 216);
            this.WaitForHelpPercentageSlider.Maximum = 100;
            this.WaitForHelpPercentageSlider.Name = "WaitForHelpPercentageSlider";
            this.WaitForHelpPercentageSlider.Size = new System.Drawing.Size(176, 45);
            this.WaitForHelpPercentageSlider.TabIndex = 5;
            this.WaitForHelpPercentageSlider.TickFrequency = 10;
            this.WaitForHelpPercentageSlider.Value = 98;
            // 
            // TestSolutionButton
            // 
            this.TestSolutionButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.TestSolutionButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.TestSolutionButton.Location = new System.Drawing.Point(616, 576);
            this.TestSolutionButton.Name = "TestSolutionButton";
            this.TestSolutionButton.Size = new System.Drawing.Size(176, 32);
            this.TestSolutionButton.TabIndex = 23;
            this.TestSolutionButton.Text = "Test Solution";
            this.TestSolutionButton.UseVisualStyleBackColor = false;
            this.TestSolutionButton.Click += new System.EventHandler(this.TestSolutionButton_Click);
            // 
            // ClearSquareButton
            // 
            this.ClearSquareButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.ClearSquareButton.Location = new System.Drawing.Point(608, 468);
            this.ClearSquareButton.Name = "ClearSquareButton";
            this.ClearSquareButton.Size = new System.Drawing.Size(24, 24);
            this.ClearSquareButton.TabIndex = 8;
            this.ClearSquareButton.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // AddTreeButton
            // 
            this.AddTreeButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.AddTreeButton.Location = new System.Drawing.Point(636, 468);
            this.AddTreeButton.Name = "AddTreeButton";
            this.AddTreeButton.Size = new System.Drawing.Size(24, 24);
            this.AddTreeButton.TabIndex = 9;
            this.AddTreeButton.Click += new System.EventHandler(this.SelectMapItemButton_Click);
            // 
            // WaterRemainingLabel
            // 
            this.WaterRemainingLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.WaterRemainingLabel.Location = new System.Drawing.Point(616, 312);
            this.WaterRemainingLabel.Name = "WaterRemainingLabel";
            this.WaterRemainingLabel.Size = new System.Drawing.Size(176, 16);
            this.WaterRemainingLabel.TabIndex = 92;
            this.WaterRemainingLabel.Text = "Water Remaining:";
            // 
            // SimulatorArea
            // 
            this.SimulatorArea.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.SimulatorArea.BackColor = System.Drawing.Color.White;
            this.SimulatorArea.Location = new System.Drawing.Point(8, 8);
            this.SimulatorArea.Name = "SimulatorArea";
            this.SimulatorArea.Size = new System.Drawing.Size(600, 600);
            this.SimulatorArea.TabIndex = 91;
            this.SimulatorArea.MouseDown += new System.Windows.Forms.MouseEventHandler(this.SimulatorArea_MouseDown);
            this.SimulatorArea.MouseMove += new System.Windows.Forms.MouseEventHandler(this.SimulatorArea_MouseMove);
            this.SimulatorArea.MouseUp += new System.Windows.Forms.MouseEventHandler(this.SimulatorArea_MouseUp);
            // 
            // AntSimulatorTab
            // 
            this.BackColor = System.Drawing.SystemColors.Control;
            this.Controls.Add(this.WaterRemainingLabel);
            this.Controls.Add(this.AddTreeButton);
            this.Controls.Add(this.SimulatorArea);
            this.Controls.Add(this.ClearSquareButton);
            this.Controls.Add(this.TestSolutionButton);
            this.Controls.Add(this.WaitForHelpPercentageLabel);
            this.Controls.Add(this.WaitForHelpPercentageSlider);
            this.Controls.Add(this.PheromoneStrengthLabel);
            this.Controls.Add(this.PheromoneDissipationSpeedLabel);
            this.Controls.Add(this.LoadSolutionCombo);
            this.Controls.Add(this.LoadSolutionLabel);
            this.Controls.Add(this.AddItemToMapLabel);
            this.Controls.Add(this.AddFood9Button);
            this.Controls.Add(this.AddFood8Button);
            this.Controls.Add(this.AddFood7Button);
            this.Controls.Add(this.AddFood6Button);
            this.Controls.Add(this.AddFood5Button);
            this.Controls.Add(this.AddFood4Button);
            this.Controls.Add(this.AddFood3Button);
            this.Controls.Add(this.AddFood2Button);
            this.Controls.Add(this.AddFood1Button);
            this.Controls.Add(this.AddObstacleButton);
            this.Controls.Add(this.AddWaterButton);
            this.Controls.Add(this.AddNestButton);
            this.Controls.Add(this.PheromoneStrengthSlider);
            this.Controls.Add(this.NumberOfAntsLabel);
            this.Controls.Add(this.ClearMapButton);
            this.Controls.Add(this.LoadMapLabel);
            this.Controls.Add(this.CannedMapComboBox);
            this.Controls.Add(this.NumberOfTurnsLabel);
            this.Controls.Add(this.FitnessLabel);
            this.Controls.Add(this.ProgramSizeLabel);
            this.Controls.Add(this.FoodNotCollectedLabel);
            this.Controls.Add(this.FoodNotLocatedLabel);
            this.Controls.Add(this.AntSpeedLabel);
            this.Controls.Add(this.AntSpeedSlider);
            this.Controls.Add(this.PheromoneDissipationSpeedSlider);
            this.Controls.Add(this.NumberOfAntsSlider);
            this.Font = new System.Drawing.Font("Verdana", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.ForeColor = System.Drawing.Color.Black;
            this.Name = "AntSimulatorTab";
            this.Size = new System.Drawing.Size(800, 616);
            ((System.ComponentModel.ISupportInitialize)(this.AntSpeedSlider)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.PheromoneDissipationSpeedSlider)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.NumberOfAntsSlider)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.PheromoneStrengthSlider)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.WaitForHelpPercentageSlider)).EndInit();
            this.ResumeLayout(false);
            this.PerformLayout();

		}
		#endregion

        /// <summary>
        /// Private reference to the main form which gets us access to the other tabs.
        /// </summary>
        private AntForm antForm;
        /// <summary>
        ///The main form which gets us access to the other tabs.
        /// </summary>
        public AntForm AntForm
        {
            set
            {
                antForm = value;
            }
            get
            {
                return antForm;
            }
        }

        /// <summary>
        /// Used to provide the custom designed map to the Evolver tab.
        /// </summary>
        /// <returns></returns>
        public Map GetMap()
        {
            return map;
        }

        /// <summary>
        /// When running in IE, call this when the user leaves the page
        /// to make sure the background threads don't keep running.
        /// </summary>
        public void StopThreads()
        {
            if (interpreter != null)
            {
                interpreter.Running = false;
                interpreter = null;
            }
        }

        /// <summary>
        /// Interpreter used to interpret the ant program.
        /// </summary>
        Interpreter interpreter = null;

        /// <summary>
        /// Event fired to test the ant program.
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
		private void TestSolutionButton_Click(object sender, System.EventArgs e)
		{
            if (program == null)
            {
                return;
            }

            if (interpreter != null)    // if not null, then we are already running, so stop the program.
            {
                interpreter.Running = false;
                interpreter = null;
                TestSolutionButton.Text = "Test Solution";
                return;
            }

            TestSolutionButton.Text = "Stop Test";

            // Start with the option settings from the evolver tab.
            options = AntForm.GetOptionsFromEvolveTab();

            // Then override the settings based on the sliders on this tab.
            options.NumberOfAnts = NumberOfAntsSlider.Value;
            options.PheromoneStrength = PheromoneStrengthSlider.Value;
            options.PheromoneDissipationSpeed = PheromoneDissipationSpeedSlider.Value;
            options.WaitForHelpThisTurnPercent = WaitForHelpPercentageSlider.Value;
            options.NumberOfTurnsPerAnt = int.MaxValue;
            options.Map = map;
            interpreter = new Interpreter();
            interpreter.TurnCompleted += new Interpreter.TurnCompletedEventHandler(interpreter_TurnCompleted);
            interpreter.Options = options;
            interpreter.Program = program;
            // Set up the event to redraw the map every turn.
            ThreadPool.QueueUserWorkItem(new WaitCallback(interpreter.ExecuteProgram));
		}

        /// <summary>
        /// Delegate for the event fired after every turn of the ant program. The delegate is required to invoke back on the GUI thread.
        /// </summary>
        /// <param name="sender">Object that generated the event.</param>
        /// <param name="e">Event arguments that contain information on the current state of the map and the ants.</param>
        public delegate void TurnCompleteEventHandler(Object sender, TurnCompleteEventArgs e);

        /// <summary>
        /// Event fired after every turn of the ant program.
        /// </summary>
        /// <param name="sender">Object that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void interpreter_TurnCompleted(object sender, TurnCompleteEventArgs e)
        {
			if (InvokeRequired)
			{
				try
				{
					this.Invoke(new TurnCompleteEventHandler(TurnComplete), new object[] { sender, e });
					Thread.Sleep((10 - e.RefreshSpeed) * 25);
					return;
				}
				catch (Exception)
				{
				}
			}
			else
			{
				TurnComplete(sender, e);
				Thread.Sleep((10 - e.RefreshSpeed) * 25);
			}
        }

        /// <summary>
        /// After every ant turn, the fitness labels are updated and the current state of the map and ants are drawn on the simulator control.
        /// </summary>
        /// <param name="sender">Object that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void TurnComplete(object sender, TurnCompleteEventArgs e)
        {
            this.NumberOfTurnsLabel.Text = "Number of Turns: " + e.Turn.ToString();
            this.FoodNotCollectedLabel.Text = "Food not Collected: " + e.FoodNotCollected.ToString();
            this.FoodNotLocatedLabel.Text = "Food not Located: " + e.FoodNotLocated.ToString();
            this.WaterRemainingLabel.Text = "Water remaining: " + e.WaterRemaining.ToString();
            this.ProgramSizeLabel.Text = "Program Size: " + e.NumberOfNodes.ToString();
            this.FitnessLabel.Text = "Fitness: " + Program.DetermineFitness(options, e.FoodNotCollected, e.FoodNotLocated, e.WaterRemaining, e.AverageAntMovement, e.NumberOfNodes);
            SimulatorArea.DrawMap(e.Map, e.Ants);
            if (e.Complete)
            {
                interpreter = null;
                TestSolutionButton.Text = "Test Solution";
            }

            // Post back the ant speed (time to sleep each turn) so that the interpreter thread knows how long to sleep this turn.
            e.RefreshSpeed = this.AntSpeedSlider.Value;
        }

        /// <summary>
        /// Event fired when the user selects a new map from the combo box.
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
		private void CannedMapComboBox_SelectedIndexChanged(object sender, System.EventArgs e)
		{
            if (CannedMapComboBox.Text.Equals("Use Same Map from Tab1"))
			{
                map = AntForm.GetMapFromEvolveTab();
			}
			else if ( CannedMapComboBox.Text.Equals( "Three Food(1) Piles" ))
			{
				map.LoadCannedMap( Map.CannedMaps.ThreeFood1Piles );
			}
			else if ( CannedMapComboBox.Text.Equals( "Six Weight Food(1,3,6) Piles" ))
			{
				map.LoadCannedMap( Map.CannedMaps.SixDifferentWeightFoodPiles );
			}
			else if ( CannedMapComboBox.Text.Equals( "Seven Food(6) Pieces" ))
			{
				map.LoadCannedMap( Map.CannedMaps.SevenFood6Pieces );
			}
			else if ( CannedMapComboBox.Text.Equals( "Ten Food(9) Pieces" ))
			{
				map.LoadCannedMap( Map.CannedMaps.TenFood9Pieces );
			}
            else if (CannedMapComboBox.Text.Equals("One Food(1) Pile With River"))
            {
                map.LoadCannedMap(Map.CannedMaps.OneFood1PileWithRiver);
            }
            else if (CannedMapComboBox.Text.Equals("Three Food(1) Piles River Tree"))
            {
                map.LoadCannedMap(Map.CannedMaps.ThreeFood1PileWithRiverAndTree);
            }
			else if ( CannedMapComboBox.Text.Equals( "Six Food(5) Pieces Wall/River" ))
			{
				map.LoadCannedMap( Map.CannedMaps.SixFood5WithWallAndRiver );
			}

			SimulatorArea.DrawMap( map );
		}


        /// <summary>
        /// Event fired to clear the map
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
		private void ClearMapButton_Click(object sender, System.EventArgs e)
		{
			map.ClearMap();
			SimulatorArea.DrawMap( map );
		}

        /// <summary>
        /// Draws the background color on one of the buttons used to place map items.
        /// If the button is selected it has a gold background.
        /// Clicking the map area will place the selected map item on that map.
        /// </summary>
        /// <param name="button">The button to set the color on.</param>
        /// <param name="selected">Is that button selected (gold background).</param>
		private void DrawMapItemButton( Button button, bool selected )
		{
			if ( selected )
			{
				button.BackColor = Color.Goldenrod;
				button.ForeColor = Color.Goldenrod;
			}
			else 
			{
				button.BackColor = Color.White;
				button.ForeColor = Color.White;
			}
		}

        /// <summary>
        /// Updates the backgrounds of all the buttons used to place map items on the map.
        /// The one selected button will have a gold background.
        /// </summary>
		private void DrawSelectedMapItem()
		{
			DrawMapItemButton( AddFood1Button, false );
			DrawMapItemButton( AddFood2Button, false );
			DrawMapItemButton( AddFood3Button, false );
			DrawMapItemButton( AddFood4Button, false );
			DrawMapItemButton( AddFood5Button, false );
			DrawMapItemButton( AddFood6Button, false );
			DrawMapItemButton( AddFood7Button, false );
			DrawMapItemButton( AddFood8Button, false );
			DrawMapItemButton( AddFood9Button, false );
			DrawMapItemButton( ClearSquareButton, false );
			DrawMapItemButton( AddNestButton, false );
			DrawMapItemButton( AddObstacleButton, false );
			DrawMapItemButton( AddWaterButton, false );
            DrawMapItemButton( AddTreeButton, false);

			if ( selectedMapItemButton != null )
			{
				DrawMapItemButton( selectedMapItemButton, true );
			}
		}

        /// <summary>
        /// Gets the Map Item associated with one of the buttons used to place Map Items on the map.
        /// The user selects one of the map buttons and then clicks the image to place that item on the map.
        /// </summary>
        /// <param name="button">The selected map item button.</param>
        /// <returns>The map item associated with the button.</returns>
		private MapItem.Item GetMapItemForButton( Button button )
		{
			if ( button == AddFood1Button ) return MapItem.Item.Food1;
			if ( button == AddFood2Button ) return MapItem.Item.Food2;
			if ( button == AddFood3Button ) return MapItem.Item.Food3;
			if ( button == AddFood4Button ) return MapItem.Item.Food4;
			if ( button == AddFood5Button ) return MapItem.Item.Food5;
			if ( button == AddFood6Button ) return MapItem.Item.Food6;
			if ( button == AddFood7Button ) return MapItem.Item.Food7;
			if ( button == AddFood8Button ) return MapItem.Item.Food8;
			if ( button == AddFood9Button ) return MapItem.Item.Food9;
			if ( button == ClearSquareButton ) return MapItem.Item.Nothing;
			if ( button == AddNestButton ) return MapItem.Item.Nest;
			if ( button == AddObstacleButton ) return MapItem.Item.Obstacle;
			if ( button == AddWaterButton ) return MapItem.Item.Water;
            if ( button == AddTreeButton) return MapItem.Item.Tree;

			return MapItem.Item.Nothing;
		}

        /// <summary>
        /// Event raised when the user selects one of the map item buttons.
        /// The selected button gets a gold border. The border of the previously selected button is reset.
        /// The next time the user click on the map, it will place this selected map item.
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
		private void SelectMapItemButton_Click(object sender, System.EventArgs e)
		{
			selectedMapItemButton = (Button) sender;
			DrawSelectedMapItem();
		}

        /// <summary>
        /// Is the user dragging the mouse. True between the mouse down and up events.
        /// </summary>
		private bool draggingMouse = false;

        /// <summary>
        /// Event raised on a mouse down event on the map area. Used to place a map item.
        /// </summary>
        /// <param name="sender">Object that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
		private void SimulatorArea_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if ( selectedMapItemButton != null )
			{
				draggingMouse = true;

				map.AddItem( e.X / ( SimulatorArea.Width / map.MapSize.Width ),
							 e.Y / ( SimulatorArea.Height / map.MapSize.Height ),
							GetMapItemForButton( selectedMapItemButton ));

				SimulatorArea.DrawMap( map );
			}
		}

        /// <summary>
        /// Event raised on a mouse move event on the map area. Used to place a map item when the user is dragging the mouse with the button down.
        /// </summary>
        /// <param name="sender">Object that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
		private void SimulatorArea_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if ( draggingMouse )
			{
				SimulatorArea_MouseDown( sender, e );
			}
		}

        /// <summary>
        /// Event raised on a mouse up event on the map area. Used to reset the draggingMouse variable.
        /// </summary>
        /// <param name="sender">Object that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
		private void SimulatorArea_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			draggingMouse = false;
		}

        /// <summary>
        /// Event raised when the user selects a new solution from the combo box.
        /// </summary>
        /// <param name="sender">Combo box that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void LoadSolutionCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (LoadSolutionCombo.Text.Equals("Simple Light Grouped Food Solution"))
            {
                program = new Program(Program.CannedPrograms.SimpleLightGroupedFoodSolution);
            }
            else if (LoadSolutionCombo.Text.Equals("Simple Heavy Food Solution"))
            {
                program = new Program(Program.CannedPrograms.SimpleHeavyFoodSolution);
            }
            else if (LoadSolutionCombo.Text.Equals("River Solution w/ Trees"))
            {
                program = new Program(Program.CannedPrograms.TreeRiverSolution);
            }
            else if (LoadSolutionCombo.Text.Equals("River Solution w/ Dead Ants"))
            {
                program = new Program(Program.CannedPrograms.DeadAntRiverSolution);
            }
            else if (LoadSolutionCombo.Text.Equals("Heavy Food with River Solution"))
            {
                program = new Program(Program.CannedPrograms.HeavyFoodwithRiverSolution);
            }
            else if (LoadSolutionCombo.Text.Equals("Use Last Evolved Solution"))
            {
                program = AntForm.GetProgramFromEvolveTab();
            }
            else if (LoadSolutionCombo.Text.Equals("Use Program From Tab 2"))
            {
                program = AntForm.GetProgramFromProgramTab();
            }
        }
	}
}
